/*************************************************************************
 *
 *	AUTHOR:	Wilson Tham
 *
 *   TITLE:	button.c
 *
 *	DATE:	April 22, 1993
 *
 *	PURPOSE: 	Query-type button definitions.
 *
 *************************************************************************/

#include <graphics.h>
#include <assert.h>
#include <string.h>
#include <stdio.h>

#include"button.h"

void PushButton::set_constants(char *name, int color, int Id)
{
	Activated		= False;
	Color		= color;
	surface		= NULL;
	strcpy(string, name);
	IdNum = Id;
	Drawn = No;
	X=0; Y=0;

	Viewport surf;
	surf.SetTextStyle(FONT, HORIZ_DIR, FONT_SIZE);

	string_height	= surf.TextHeight(string);
	// Adjust for differing fonts...
	string_height	= (int)(string_height+(string_height*.33));
	string_width	= surf.TextWidth(string);
}

void PushButton::Init(Display *d, int x, int y)
{
	X = x; X2+=X;
	Y = y; Y2+=Y;

	Disp = d;
}

void PushButton::Draw()
{
	Activated	= False;
	Drawn	= Yes;

	Disp->FastDraw();
	Disp->SetFillStyle(SOLID_FILL, LIGHTGRAY);
	Disp->Bar(X+FRAME, Y+FRAME, X2-FRAME, Y2-FRAME);

	draw_shadows(BOTTOM_RIGHT, DARKGRAY);
	draw_shadows(TOP_LEFT, WHITE);
	draw_frame();
	name_button();
	Disp->Done();
}

void PushButton::Erase()
{
	if(Drawn == Yes) {
		Drawn = No;
		Disp->SetFillStyle(SOLID_FILL, Disp->GetBkColor());
		Disp->Bar(X, Y, X2, Y2);
	}
}

void PushButton::redraw()
{
	Disp->FastDraw();

	draw_shadows(BOTTOM_RIGHT, DARKGRAY);
	draw_shadows(TOP_LEFT, WHITE);
	name_button();
	Disp->Done();
}

void PushButton::depress_button()
{
	Disp->FastDraw();

	// Re-modified for test...
	draw_shadows(BOTTOM_RIGHT, LIGHTGRAY);
	draw_shadows(TOP_LEFT, LIGHTGRAY);
	Disp->SetColor(DARKGRAY);
	Disp->Line(X+FRAME, Y+FRAME, X2-FRAME, Y+FRAME);
	Disp->Line(X+FRAME, Y+FRAME, X+FRAME, Y2-FRAME);

	name_button();
	Disp->Done();
}

void PushButton::draw_shadows(SHADOW_STYLE style, int color)
{
	// Side shadows...
	int x = X+FRAME, y = Y+FRAME, x2 = X2-SHADOWS+FRAME, y2 = Y2-SHADOWS+FRAME;

	Disp->SetColor(color);
	for(int count=0; count<SHADOWS; count++)
	{
		switch(style) {
			case TOP_LEFT:
				Disp->Line(x+count, y+count, x2-count, y+count);
				Disp->Line(x+count, y+count, x+count, y2-count);
			break;
			case BOTTOM_RIGHT:
				Disp->Line(x+count, y2-count, x2-count, y2-count);
				Disp->Line(x2-count, y+count, x2-count, y2-count);
			break;
			case TOP_RIGHT:
				// Undefined for now...
			break;
			case BOTTOM_LEFT:
				// Undefined for now...
			break;
		}
	}
}

BOOLEAN PushButton::ButtonBounds(int Mx, int My)
{
	// Mouse will call and supply the coordinates of its position
	//	It's up to the button itself to determine it's bounds.

	BOOLEAN flag = False;

	if ((Mx<=X2-SHADOWS) && (Mx>=X) && (My<=Y2-SHADOWS) && (My>=Y))
		flag = True;
	else
		flag = False;

	return flag;
}

void PushButton::draw_frame()
{
	Disp->SetLineStyle(SOLID_LINE, 0, FRAME+1);
	Disp->SetColor(BLACK);
	Disp->Rectangle(X, Y, X2, Y2);
	Disp->SetLineStyle(SOLID_LINE, 0, 1);
}

void PushButton::name_button()
{
	int displacement, x, y, x2, y2;

	displacement = (Activated)? 1 : 0;

	x	= X+BORDERX+SHADOWS+FRAME+displacement+1;
	y	= Y+BORDERY+SHADOWS+FRAME+displacement-2;
	x2	= X2-BORDERX-SHADOWS-FRAME+displacement+1;
	y2	= Y2-BORDERY-SHADOWS-FRAME+displacement+1;

	Disp->SetColor(LIGHTGRAY);
	Disp->Rectangle(x-displacement, y-displacement, x2+1, y2+1);

	Viewport surf (Disp, x, y, x2, y2);

	if(surface==NULL) {
		int s = imagesize(x, y, x2, y2);
		surface = (void*) new byte [s];
		assert (surface != NULL);
		surf.SetColor(Color);
		surf.SetTextStyle (FONT, HORIZ_DIR, FONT_SIZE);
		surf.SetTextJustify(CENTER_TEXT, CENTER_TEXT);
		surf.OutTextXY(surf.GetMaxX()/2, surf.GetMaxY()/2, string);
		surf.GetImage(0, 0, surf.GetMaxX(), surf.GetMaxY(), surface);
	}
	else
		surf.PutImage(0, 0, surface, COPY_PUT);
}

Message PushButton::Dispatch(Message &msg)
{
	switch(msg.origin) {
		case MouseMsg:
			if (ButtonBounds(msg.d1, msg.d2)) {
				switch(msg.code) {
					case LMouseDown:
						Activated = True;
						animate_button();
						return Acknowledged;
					case MouseStillDown:
						if(!Activated) {
							Activated = True;
							animate_button();
						}
						return Acknowledged;
					case LMouseUp:
						Activated = False;
						animate_button();
						return Message (ButtonMsg, ButtonPress, IdNum);
					default:
						return NullMsg;
				}
			}
			if(msg.code==MouseStillDown && Activated) {
				Activated = False;
				animate_button();
			}
			return NullMsg;

/*		case KeyMsg:
			if (KeyBounds(msg.d1, msg.d2)
			return NullMsg;						*/
	}

return NullMsg;
}