/*************************************************************************
 *
 *	AUTHOR:	Wilson Tham
 *
 *	TITLE:	group.c
 *
 *	DATE:	April 28, 1993
 *
 *	PURPOSE:  Definition of groups of checkboxes.
 *
 *************************************************************************/

#include "group.h"

Group::~Group()
{
	for(int count=0; count<CurrentObj; count++)
		delete Boxes[count];

	free (Boxes);
}

void Group::Add(CheckBox *newbox, int x, int y)
{
	if(CurrentObj!=NumObj) {
		Boxes[CurrentObj] = newbox;
		newbox->Init(Disp, x, y);

		CurrentObj++;
	}
}

Message Group::Dispatch(Message &msg)
{
	int count=0;
	Message msg2;

	do{
		if((msg.origin==GodMsg)&&(msg.code==Disable)&&(msg.d1==IdNum))
			disable_proc();
		if((msg.origin==GodMsg)&&(msg.code==Enable)&&(msg.d1==IdNum))
			enable_proc();

		msg2=Boxes[count]->Dispatch(msg);
		if (msg2.code==Status) {
			switch(msg2.d2) {
				case On:
					off_last();
					break;
				case Off:
					if (CurrentId==count)
						Boxes[CurrentId]->ChangeStatus(On);
					break;
			}
			CurrentId=count;
		}
		count++;
	}while((count<CurrentObj)&&(msg2!=Acknowledged)&&(msg2.code!=Status));

	return msg2;
}

void Group::Draw()
{
	Disp->FastDraw();
	for (int count=0; count<CurrentObj; count++)
		Boxes[count]->Draw();

	// Set Default button On...
	Boxes[Default]->ChangeStatus(On);
	Disp->Done();
}

void Group::Erase()
{
	Disp->FastDraw();
	for (int count=0; count<CurrentObj; count++)
		Boxes[count]->Erase();
	Disp->Done();
}

void Group::DisableBox(int idnum)
{
	int iteration;
	iteration=id_to_int(idnum);

	Boxes[iteration]->Dispatch (Message (GodMsg, Disable, idnum));

	if(Default==CurrentObj)
		Default=-1;

	if(Default==iteration) {
		Default=iteration+1;
		CurrentId=iteration+1;
		Boxes[iteration]->ChangeStatus(Off);
		Boxes[iteration]->Dispatch (Message (GodMsg, Disable, idnum));
		Boxes[Default]->ChangeStatus(On);
	}
}

void Group::EnableBox(int idnum)
{
	int iteration;
	iteration=id_to_int(idnum);

	Boxes[iteration]->Dispatch (Message (GodMsg, Enable, idnum));

	if(iteration==0) {
		Default=0;
		CurrentId=0;
	}
}

int Group::id_to_int(int idnum)
{
	int count=0;

	while (Boxes[count]->BoxId()!=idnum)
		count++;

	return count;
}

void Group::disable_proc()
{
	int idnum;

	Disp->FastDraw();
	for(int count=0; count<CurrentObj; count++) {
		idnum=Boxes[count]->BoxId();
		Boxes[count]->Dispatch (Message (GodMsg, Disable, idnum));
	}

	Disp->Done();
}

void Group::enable_proc()
{
	int idnum;

	Disp->FastDraw();
	for(int count=0; count<CurrentObj; count++) {
		idnum=Boxes[count]->BoxId();
		Boxes[count]->Dispatch (Message (GodMsg, Enable, idnum));
	}

	// Set Default button On...
	Boxes[Default]->ChangeStatus(On);

	Disp->Done();
}