/**********************************************************************
*       FILE:     MAPNODE.c
*       AUTHOR:   E. Otis Owens
*       DATE:     17 April 1993
*       MODIFIED: 30 April 1993 (5:00 pm)
*       PURPOSE:
*         Functions to store and manipulate a node.
***********************************************************************/

#include <assert.h>
#include "mapnode.h"


/**********************************************************************
*       Public Functions
***********************************************************************/

bool MapNode::is_link (mode transportation) {

  int i= 0;

  // if bit vector "transportation" is on...
  while ( (i < numLinks) && !(links[i].type & On << transportation) ) i++;
  // ...then "i" should be less than numLinks so return True, else False
  return (i < numLinks) ? True : False;
}

/************************************************************/

bool MapNode::is_link (int link, mode transportation) {
// Determine if node is a taxi link

  int i= 0;
 
  while (link != links[i].destination) i++;  // find the link...
  // ...and return whether its transportation bit is on or off
  return (links[i].type & On << transportation);
}

/************************************************************/

vect MapNode::find_vector (int link) {
// Return the bit vector of a given link

  int i= 0;

  while (link != links[i].destination) i++;  // find the link...
  // ...and return its bit vector
  return links[i].type;
}

/************************************************************/

int MapNode::find_link (int linkNum) {
  if ( between(linkNum, 0, numLinks) ) return links[linkNum].destination;
  else {
    cerr << "Link " << linkNum << " out of range ("
         << 0 << ".." << numLinks << ")\n";
    return 0;  // node 0 points to NULL
  }
}


/************************************************************/

void MapNode::get_name (char *nodeName, int size) {
// Input a node name (if one exists) of size "size"

  if (size > 1) {
    name= new char[size];
    for (int i= 0; i < size; i++) name[i]= nodeName[i];
  }
}

/*************************************************************/

void MapNode::get_link (int node, mode transportation) {
// Input a link and establish its "transportation" type in the bit
// vector

  Route *oldLinks= links;
  int   num= 0;

  if (numLinks > 0)
    while ((node != links[num].destination) && (num < numLinks)) num++;

  if (num >= numLinks) {
    num= numLinks;

    links= new Route[++numLinks];
    assert( links != 0 );

    for (int i= 0; i < num; i++)
      links[i]= oldLinks[i];  // copy the array 

    links[num].destination= node;
  }
  // turn transportation bit on
  links[num].type |= On << transportation;
}

/*************************************************************/

void MapNode::show_node (void) {
// Output a node in a tabular (sort of) format for debugging purposes

  int i;

  cout << "\nTaxi: ";
  for (i= 0; i < numLinks; i++)  // find all taxi routes
    if (links[i].type & On << Taxi) cout << ' ' << links[i].destination;
  cout << "\nBus: ";
  for (i= 0; i < numLinks; i++)  // find all bus routes
    if (links[i].type & On << Bus) cout << ' ' << links[i].destination;
  cout << "\nSubway: ";
  for (i= 0; i < numLinks; i++)  // find all subway routes
    if (links[i].type & On << Subway) cout << ' ' << links[i].destination;
  cout << "\nAll:  ";
  for (i= 0; i < numLinks; i++)  // show all routes and (bit vector)s
    cout << links[i].destination << " (" << links[i].type << ") ";
  cout << "\nName: ``";
  if (name == NULL) cout << "(null)";
  else cout << name;
  cout << "''\n";
  cout << "x: " << x << "   y: " << y << "\n";
}










