/**********************************************************************
*    FILE:     players.h
*    AUTHOR:   CALeague
*    DATE:     29 April 1993
*    PURPOSE:
*      Declares the player hierarchy.
***********************************************************************/

#ifndef _PLAYERS_H
#define _PLAYERS_H

#include <values.h>
#include "moves.h"

typedef int PosType;

const int Invalid = MAXINT;
const int NameSize = 40;
const int MaxPlayers = 6;
const int theSpy = 0;          // thePlayers [theSpy]

/**********************************************************************
*    CLASS:  Player
***********************************************************************/

class Player
{
  char Name [NameSize];
  int  Color;

protected:
  PosType Pos;
  MoveList History;
  virtual Move ChooseMove (void) { return NullMove; }
  Player (void) { }

public:
  Player (char* name, int color);
  Player (char* name, int color, PosType pos);
  virtual ~Player (void) { }

  PosType Position (void) { return Pos; }
  PosType GetPosition (void) { return Pos; }
  virtual bool IsSpy (void);
  char* GetName (void) { return Name; }
  int GetColor (void) { return Color; }
  MoveList& GetHistory (void) { return History; }

  virtual int NumTokens (TokenType) { return 0; }
  virtual Move TakeTurn (void) { return NullMove; }
};


/**********************************************************************
*    CLASS:  Detective
***********************************************************************/

class Detective : virtual public Player
{
  int Tokens [NTokenTypes];

public:
  Detective (void);
  bool IsSpy (void) { return False; }
  int NumTokens (TokenType t) { return (Tokens [t]); }
  Move TakeTurn (void);
};


/**********************************************************************
*    CLASS:  Spy
***********************************************************************/

class Spy : virtual public Player
{
public:
  bool IsSpy (void);
  int NumTokens (TokenType) { return Invalid; }
  Move TakeTurn (void);
  MoveList& GetHistory (void) { return History; }
};


/**********************************************************************
*    CLASS:  Human
***********************************************************************/

class Human : virtual public Player
{
protected:
  Move ChooseMove (void);
};


/**********************************************************************
*    CLASS: Computer
***********************************************************************/

class Computer : virtual public Player { };


/**********************************************************************
*    CLASS: HumanDetective
***********************************************************************/

class HumanDetective : public Human, public Detective
{
public:
  HumanDetective (char* name, int color) : Player (name, color) { }
  HumanDetective (char* name, int color, PosType pos)
    : Player (name, color, pos) { }
};


/**********************************************************************
*    CLASS: HumanSpy
***********************************************************************/

class HumanSpy : public Human, public Spy
{
public:
  HumanSpy (char* name, int color) : Player (name, color) { }
  HumanSpy (char* name, int color, PosType pos)
    : Player (name, color, pos) { }
};


/**********************************************************************
*    CLASS: ComputerDetective
***********************************************************************/

class ComputerDetective : public Computer, public Detective
{
protected:
  Move ChooseMove (void);
  byte NumNodeTypes (int node);
  bool CanLeave (Move *duh);
  Move *FindNearest (int x, int y, MoveList *moves);
public:
  ComputerDetective (char* name, int color) : Player (name, color) { }
  ComputerDetective (char* name, int color, PosType pos)
    : Player (name, color, pos) { }
};


/**********************************************************************
*    CLASS: ComputerSpy
***********************************************************************/

class ComputerSpy : public Computer, public Spy
{
protected:
  Move ChooseMove (void);
  Move *FindFarthest (int x, int y, MoveList *moves);
public:
  ComputerSpy (char* name, int color) : Player (name, color) { }
  ComputerSpy (char* name, int color, PosType pos)
    : Player (name, color, pos) { }
};


/**********************************************************************
*    Global array of players
***********************************************************************/

extern Player* thePlayers [MaxPlayers];
extern int NumPlayers;


#endif
