// ********************************************************************
// * AUTHOR: Steven Winig					      *
// * TITLE: UFWOOM.C                                                  *
// * DATE: April 29, 1993                                             *
// * PURPOSE: User-Friendly Windows Oriented Options Menu             *
// ********************************************************************

#include "ui.h"
#include "ufwoom.h"
#include "group.h"
#include "button.h"
#include "window.h"
#include <conio.h>
#include <stdio.h>
#include <alloc.h>
#include <assert.h>



int* UserInterface::UFWOOM()
{
  Display UFWOOMDisplay;
  int* info; // used to return info to user

  int Num_Buttons = 0;
  int Num_Square_CheckBoxes = 0;
  int Num_Circle_CheckBoxes = 0;
  int Num_Groups = 0;

  char buf[40];

  int i;
  info = new int[6]; // new space to store info

  UFWOOMDisplay.SetBkColor (BLACK);
  UFWOOMDisplay.Clear ();

  CheckBox *box [MaxPlayers*3]; // each player requires three checkboxes
  for (int plyr = 0; plyr < MaxPlayers; plyr++)
  { // new control checkboxes for each player
    box[plyr*2] = new CheckBox ("Computer", BLACK, PColor[plyr], Initial_Square_CheckBox_ID + Num_Square_CheckBoxes, Square);
    Num_Square_CheckBoxes++; // increment square check box count
    box[(plyr*2)+1] = new CheckBox ("Human", BLACK, PColor[plyr], Initial_Square_CheckBox_ID + Num_Square_CheckBoxes, Square);
    Num_Square_CheckBoxes++;
  }

  box[Num_Square_CheckBoxes + Num_Circle_CheckBoxes] = new CheckBox ("Mr. X", BLACK, PColor[0], Initial_Circle_CheckBox_ID + Num_Circle_CheckBoxes, Circle);
  Num_Circle_CheckBoxes++;

  for (i = 0; i < MaxPlayers; i++) {
    sprintf (buf, "Detective %d", i+1);
    box[Num_Square_CheckBoxes + Num_Circle_CheckBoxes] = new CheckBox (buf, BLACK, PColor[i+1], Initial_Circle_CheckBox_ID + Num_Circle_CheckBoxes, Circle);
    Num_Circle_CheckBoxes++;
    }

  PushButton *pb = new PushButton ("Play Game", BLUE, Initial_Button_ID);
  Num_Buttons++;

  // assert gracefully aborts if failure
  for (i=0; i < (Num_Square_CheckBoxes + Num_Circle_CheckBoxes); i++)
    assert (box[i] != NULL);
  assert (pb != NULL);

  Group *g[6];

  for (i=0; i < MaxPlayers; i++)
  {
    g[i] = new Group (2, Initial_Group_ID + Num_Groups);
    Num_Groups++;
  }

  Window *w = new Window (376, 235, RED, LIGHTGRAY, "Player Information", WHITE);
  assert (w != NULL);
  // add window to display
  UFWOOMDisplay.Add (w, UFWOOMDisplay.GetMaxX()/2-(376/2), UFWOOMDisplay.GetMaxY()/2-(235/2));
  for(i=0; i<MaxPlayers; i++)
    w->Client().Add (g[i], 0, 0); // add groups to the window

  for (i=0; i<12; i+=2) {
    g[i/2]->Add(box[i], 167, 40+(20*(i/2))); // add boxes to the groups
    g[i/2]->Add(box[i+1], 267, 40+(20*(i/2)));
  }

  for (i=0; i < MaxPlayers; i++) // add boxes to windows
    w->Client().Add (box[i+12], 10, 40+(20*i));
  // add push button to window
  w->Client().Add (pb, (w->Client().GetMaxX()/2)-(pb->GetXDim()/2), w->Client().GetMaxY() - pb->GetYDim() - 5);

  DrawTitleScreen(); // this probably shouldn't be in func UFWOOM, but for now

  UFWOOMDisplay.Draw ();

  w->Client().SetTextStyle(DEFAULT_FONT, 0, 1);
  w->Client().SetColor(WHITE);
  w->Client().OutTextXY(15, 20, "Players");
  w->Client().OutTextXY(170, 20, "Control");

  for(i=102; i<107; i++) // disable circle buttons for all but spy and player 1
    UFWOOMDisplay.Dispatch (Message (GodMsg, Disable, i-100+200));

// the circle button for the spy and detective 1 are ALWAYS on
  box[12] -> ChangeStatus (On); // A game consists of a spy
  box[12] -> SetPseudo (Off);
  box[13] -> ChangeStatus (On); // and at least 1 detective
  box[13] -> SetPseudo (Off);

  Message DoneMsg (ButtonMsg, ButtonPress, 0xFF), m;

  do {
    m = UFWOOMDisplay.Dispatch (Mouse.Event ());

    if ((m.origin==ButtonMsg)&&(m.code==Status)&&(m.d2)) {
      for(i=102; i<107; i++) {
	if(m.d1==i)
	  UFWOOMDisplay.Dispatch (Message (GodMsg, Enable, i-100+200));
	 }
    }
    else
    if ((m.origin == ButtonMsg) && (m.code == Status)&&(!m.d2)) {
      for(i=102; i<107; i++) {
	if(m.d1==i)
	  UFWOOMDisplay.Dispatch (Message (GodMsg, Disable, i-100+200));
	 }
    }
  } while (m != DoneMsg); // when they click Play Game button, end loop

  // loop through the circle buttons and request their status
 for (i = Initial_Circle_CheckBox_ID; i < Initial_Circle_CheckBox_ID + Num_Circle_CheckBoxes; i++) {
   m = UFWOOMDisplay.Dispatch (Message (GodMsg, RequestStatus, i));
   info[i - Initial_Circle_CheckBox_ID] = Inactive; // initialize players to inactive
   if ((m.origin == ButtonMsg) && (m.code == Status) && (m.d1 == i))
   {
    if (m.d2) // button is pressed
    { // are they controlled by a human or the computer
      m = UFWOOMDisplay.Dispatch (Message (GodMsg, RequestStatus, (Initial_Square_CheckBox_ID + ((i - Initial_Circle_CheckBox_ID) *2))));
      if ((m.origin == ButtonMsg) && (m.code == Status) && (m.d1 == Initial_Square_CheckBox_ID + ((i - Initial_Circle_CheckBox_ID) *2)))
      { // note: control buttons are mutually exclusive
	if (m.d2) // they are controlled by the computer
	  {info[i - Initial_Circle_CheckBox_ID] = ComputerP;}
	else if (!m.d2) // they are controlled by a human
	  {info[i - Initial_Circle_CheckBox_ID] = HumanP;}
      }
    }
   }
 }
 return (info);
}


