/*************************************************************************
* 	FILE:	window.h
*	AUTHOR: CALeague
*	DATE:	25 April 1993
*	PURPOSE:
*	  A window is a rectangular screen object which has a border and
*	  may have a title bar.  The area contained inside the border and
*	  title bar is called the client area.  The client area of a window
*	  is, itself, a Display.  This means that you can draw on it
*	  and it can contain ScreenObjects, such as buttons and windows.
**************************************************************************/

#ifndef _WINDOW_H
#define _WINDOW_H

#include "display.h"


/**********************************************************************
*	Constants
***********************************************************************/

const int BorderWidth 	= 2;
const int TitleWidth 	= 11;
const int TitleFont 	= SMALL_FONT;
const int TitleFontSize = 4;


/**********************************************************************
*	Window class
***********************************************************************/

class Window : public ScrnObj
{
protected:
  int XDim, YDim;			// Dimensions of window
  int CliX1, CliY1, CliX2, CliY2;	// Client area dimensions
  int BorderColor, BkColor;
  Display *ClientD;			// Client area of window

  int TitleColor;			// Title stuff
  int TX1, TY1, TX2, TY2;
  char* Title;
  Display *TitleD;

  Window ();
public:
  Window (int dx, int dy, int bc, int bkc);	// Normal window
  Window (int dx, int dy, int bc, int bkc, char* t, int tc);	// Title
  virtual ~Window ();

  int GetXDim (void) { return XDim; }
  int GetYDim (void) { return YDim; }

  Display& Client (void) { return (*ClientD); }

  void CalcBorder (void);
  void CalcTitle (void);

  virtual void PrepScreen (void) { }
  void DrawBorder (void);
  void DrawTitle (void);

  virtual void Init (Display *d, int x, int y);
  virtual void Draw ();
  virtual void Erase ();

  virtual Message Dispatch (Message& msg);
};


#endif
